//bmp.h

#ifndef _BMP_H_
#define _BMP_H_

#include <sys/types.h>
#include <stdint.h>
#include <stdio.h>

//#define _version_h      "V 2.0"
#define _BMP_version_h		0x151230        //год:месяц:число

#pragma pack(1)

#define FontNum_8_8_1 		881
#define FontBits_8_8 		8
#define FontByts_8_8 		8
#define FontLR_8_8_1 		0xFF
#define FontNBit_8_8 		0

#define FontNum_5_7_1 		571
#define FontLR_5_7_1 		0
#define FontBits_5_7 		5
#define FontByts_5_7 		7
#define FontNBit_5_7 		0//3


//В начале стоит заголовок файла (BITMAPFILEHEADER). Он описан следующим образом:
#pragma pack(1)
typedef struct tagBITMAPFILEHEADER {
    uint16_t	bfType;
    uint32_t	bfSize;
    uint16_t	bfReserved1;
    uint16_t	bfReserved2;
    uint32_t	bfOffBits;
} BITMAPFILEHEADER, *PBITMAPFILEHEADER;


//А дальше идет структура BITMAPINFOHEADER, которая объявлена так:
#pragma pack(1)
typedef struct tagBITMAPINFOHEADER {
    uint32_t	biSize;
    uint32_t		biWidth;
    uint32_t		biHeight;
    uint16_t	biPlanes;
    uint16_t	biBitCount;
    uint32_t	biCompression;
    uint32_t	biSizeImage;
    uint64_t	biXPelsPerMeter;
    uint64_t	biYPelsPerMeter;
    uint32_t	biClrUsed;
    uint32_t	biClrImportant;
} BITMAPINFOHEADER, *PBITMAPINFOHEADER;

//позиция версии структуры
#define _bcSize          0x0E

typedef struct tagBITMAPINFOHEADERr {
    uint32_t	biSize;
    uint8_t		data[128];
} BITMAPINFOHEADERr, *PBITMAPINFOHEADERr;

#pragma pack(1)
typedef struct tagRGBQUAD {
    uint8_t	rgbBlue;
    uint8_t	rgbGreen;
    uint8_t	rgbRed;
    uint8_t	rgbReserved;
} RGBQUAD;

#pragma pack(1)
typedef struct _BMP_PAR {
    uint32_t		step_x;
    uint32_t		step_y;
} BMP_PAR;

#define _BMP_DUMP_SIZE_     0x100000//0x200000
extern uint8_t BMP_DUMP[_BMP_DUMP_SIZE_];
extern uint8_t *BmpDump;
#define _interval_redraw            500000
extern uint8_t BMP_REDRAW_WORK;      //поток перезаписи
extern uint8_t BMP_FILE;

extern BITMAPFILEHEADER		bhfi;
extern BITMAPINFOHEADER		bhfo;
extern BITMAPINFOHEADERr	bhfor;

extern pthread_mutex_t         	BMPmutex;
//===регион===
extern int 		x1_clip;
extern int 		y1_clip;
extern int 		x2_clip;
extern int 		y2_clip;
//===font==
extern int		FontBits;
extern int		FontByts;
extern int		FontNBit;
extern uint8_t		FontLR;
extern uint8_t*		AddrFont;
//
#define BMP_RAD		57.2957795131
/*
***************************************
* функции статуса изображения         *
***************************************
*/
extern int BMP_FileAnalise(char *fname, uint8_t *bd);

/*
*****************************************************
* функции прямого обращения к элементам изображению *
*****************************************************
*/
extern int BMP_SetPixel(int16_t x, int16_t y, uint8_t R, uint8_t G, uint8_t B);
extern __inline int BMP_GetPixel(int16_t x, int16_t y, uint8_t * R, uint8_t * G, uint8_t * B);
extern __inline int BMP_XchangePixel(int16_t nX, int16_t nY, uint8_t R, uint8_t G, uint8_t B, uint8_t BR, uint8_t BG, uint8_t BB);
extern __inline int iBMP_SetPixel(int16_t x, int16_t y, uint8_t R, uint8_t G, uint8_t B);
/*
*****************************************************
* функции примитивов  (формирования фигур/элементов)*
*****************************************************
*/
//линия (отрезок) 
extern __inline void BMP_Line(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B);
extern __inline void BMP_HLinePX(int16_t x, int16_t y, int16_t len, uint8_t R, uint8_t G, uint8_t B);
//дуга
extern __inline void BMP_arc(int16_t x, int16_t	y, int16_t r, int16_t fi1, int16_t fi2, uint8_t R, uint8_t G, uint8_t B);
//сектор
extern __inline void BMP_sect(int16_t x, int16_t y, int16_t r, int16_t scnt, int16_t fi1, int16_t fi2, uint8_t R, uint8_t G, uint8_t B);
//окружность-круг
extern __inline void BMP_Circle(int16_t x, int16_t y, int16_t radius, uint8_t R, uint8_t G, uint8_t B);
extern __inline void BMP_Circlefill(int16_t x, int16_t y, int16_t radius, uint8_t R, uint8_t G, uint8_t B);
extern __inline void BMP_Circlefill_fuzzy(int16_t x, int16_t y, int16_t radius, uint8_t R, uint8_t G, uint8_t B, int f);
//прямоугольник
extern __inline void BMP_rect(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B);
extern __inline void BMP_rect_filled(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B);
extern __inline void BMP_rectXchange(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B, uint8_t BR, uint8_t BG, uint8_t BB);
extern __inline void BMP_RectFilledPX(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B);
//треугольник
extern __inline void BMP_triangle(int16_t x1, int16_t  y1, int16_t x2, int16_t y2, int16_t x3, int16_t y3, uint8_t R, uint8_t G, uint8_t B);
//текст
extern __inline uint8_t BMP_SetFont(int num);
extern __inline void BMP_outchar(char sym, int16_t x, int16_t y, uint8_t R1,  uint8_t G1, uint8_t B1, uint8_t R2, uint8_t G2, uint8_t B2, uint8_t FBackColor);
extern __inline void BMP_textout(char *str, int16_t x, int16_t y, uint8_t RF, uint8_t GF, uint8_t BF, uint8_t RB, uint8_t GB, uint8_t BB,  uint8_t FB);
extern __inline void BMP_outcharV(char sym, int16_t x, int16_t y, uint8_t R1,  uint8_t G1, uint8_t B1, uint8_t R2, uint8_t G2, uint8_t B2, uint8_t FBackColor);
extern __inline void BMP_textoutV(char *str, int16_t x, int16_t y, uint8_t RF, uint8_t GF, uint8_t BF, uint8_t RB, uint8_t GB, uint8_t BB,  uint8_t FB);

/***************************
функции работы с регионом
****************************/

extern __inline void BMP_GSetClip(int x1, int y1, int x2, int y2);


/*
****************************************************
* служебные функции                                *
****************************************************
*/
extern  void pthBMP_Redraw(void);


#endif
