//mrtu.c
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "mrtu.h"
#include <sys/types.h>

int	fd;
int	wait_tx_rx;
struct termios serial;
u_int8_t	rxbuf[_szbuf];
u_int8_t	txbuf[_szbuf];

inline u_int16_t MRTU_CRC(u_int8_t * data, u_int16_t len)
{
        u_int16_t	res_CRC = 0xFFFF;
        u_int16_t	count = 0;
        u_int8_t	count_crc;
        u_int8_t	dt;
        while(count < len)
        {
                //
                count_crc = 0;
                dt = (u_int8_t)(data[count]);
                res_CRC ^= (ushort)(dt);
                //
                while(count_crc < 8)
                {
                        if((res_CRC & 0x0001) < 1)
                        {
                                res_CRC = (res_CRC >> 1) & 0x7FFF;
                        }
                        else
                        {
                                res_CRC = (res_CRC >> 1) & 0x7FFF;
                                res_CRC ^= 0xA001;
                        };
                        count_crc++;
                };
                count++;
        }
        return (res_CRC);
}


int MRTU_Connect(char* strPort, int baud, int stout)
{
	//struct termios tio;
	int i, table_size= 23;
	//===== speed tables ======
	int speed_table1[]={ 0, 50, 75, 110, 134, 150, 200, 300,
			     600, 1200, 1800, 2400, 4800, 9600,
			     19200, 38400, 57600, 115200, 230400,
			     460800, 500000, 576000, 921600};
	int speed_table2[]={ B0, B50, B75, B110, B134, B150, B200, B300,
			     B600, B1200, B1800, B2400, B4800, B9600,
			     B19200, B38400, B57600, B115200, B230400,
			     B460800, B500000, B576000, B921600};
	//=========================
	for( i= 1; i< table_size; i++)	///< i start from 1, bellow 50 will be set to B0
		if( speed_table1[ i] >= baud)
			break;
	//fd = open(strPort, O_RDWR | O_NOCTTY | O_NDELAY);
	fd = open(strPort, O_RDWR);
	//
	if (fd == -1) {
        	perror(strPort);
        	return -1;
    	}
	//
	if (tcgetattr(fd, &serial) < 0) {
        	perror("Getting configuration");
        	return -1;
    	}
	//serial= fd;
	bzero( &serial, sizeof(serial));		///< clear struct for new port settings
	// Set up Serial Configuration
    	serial.c_iflag = 0;
    	serial.c_oflag = 0;
    	serial.c_lflag = 0;
    	serial.c_cflag = 0;
 
    	serial.c_cc[VMIN] = 0;
    	serial.c_cc[VTIME] = 1;
 
    	//serial.c_cflag = B9600| CS8 | CREAD;
	serial.c_cflag = speed_table2[ i] | CS8 | CREAD;
 	tcflush( fd, TCIFLUSH);
	cfsetispeed( &serial, speed_table2[ i]);
	cfsetospeed( &serial, speed_table2[ i]);
    	tcsetattr(fd, TCSANOW, &serial); // Apply configuration
	tcflush(fd, TCIOFLUSH);
	tcsetattr(fd, TCSANOW, &serial); // Apply configuration
	//
	wait_tx_rx = stout;
	//
	return 0;
}

int MRTU_Disconnect(void)
{
	close(fd);
	return 0;
}

void MRTU_TestRxTx(u_int8_t * sd, int sl, u_int8_t * rd, int rl)
{
	int	cnt;
	cnt	= 0;
	printf("send bytes: ");	
	while(cnt < sl)
	{
		printf("%02X ", *(sd + cnt));
		cnt++;
	}
	cnt	= 0;
	printf("\nrecv bytes: ");	
	while(cnt < rl)
	{
		printf("%02X ", *(rd + cnt));
		cnt++;
	}
	printf("\n");
}

//команда 3 - чтение значения регистров
int MRTU_CMD_3(u_int8_t Dev, u_int16_t Addr, u_int16_t * Reg, u_int16_t CntReg)
{
	u_int16_t		crc;
	u_int16_t		incrc;
	u_int16_t		reg;
	int 		wcount;
	int 		rcount;
	int		bsz;
	int		bcnt;
	int		cntrg;
	//
	txbuf[0]	= Dev;	//адрес
	txbuf[1]	= 3;	//команда
	txbuf[2]	= (u_int8_t)((Addr & 0xFF00) >> 8);
	txbuf[3]	= (u_int8_t)(Addr & 0xFF);
	txbuf[4]	= (u_int8_t)((CntReg & 0xFF00) >> 8);
	txbuf[5]	= (u_int8_t)(CntReg & 0xFF);
	//
	crc 		= MRTU_CRC(&txbuf[0], 6);
	txbuf[6]	= (u_int8_t)((crc >> 8) & 0xFF);
	txbuf[7]	= (u_int8_t)(crc & 0xFF);
	//
	tcflush(fd, TCIOFLUSH);
	wcount = write(fd, &txbuf[0], 8);
    	if (wcount < 8) {
		#if(_mdbg == 1)
        		printf("send CMD3 error! ...dev: %i  addr: %i\n", Dev, Addr);
		#endif
        	return _tx_err;
    	}
	//tcflush(fd, TCIOFLUSH);
	usleep(wait_tx_rx);
	memset(&rxbuf[0], 0, _szbuf);
	fcntl(fd, F_SETFL, FNDELAY);
	rcount = read(fd, &rxbuf[0], _szbuf);
	//
    	if (rcount < 1)
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD3 error! ...dev: %i  addr: %i\n", Dev, Addr);
		#endif
        	return _rx_err;
	}
	crc		= MRTU_CRC(&rxbuf[0], rcount - 2);
	incrc 		= (u_int16_t)((u_int8_t)(rxbuf[rcount - 2])) << 8;
	incrc 		|= (u_int16_t)((u_int8_t)(rxbuf[rcount - 1]));
	if (crc != incrc)
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD3 CRC error! ...dev: %i  addr: %i  rcount: %i  CRC: %04X  INCRC: %04X\n", 
			Dev, Addr, rcount, crc, incrc);
		#endif
        	return _rx_crc_err;
	}
	//проверяем совпадение команд исключая ошибки их выполнения
	if(txbuf[1] != (rxbuf[1] & 0x7F))
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD3 no main CMD! ...dev: %i  addr: %i sendCMD: %02X recvCMD: %02X\n", 
				Dev, Addr, txbuf[1], (rxbuf[1] & 0x7F));
			MRTU_TestRxTx(&txbuf[0], 8, &rxbuf[0], rcount);
		#endif
		return _rx_cmd_err;
	}
	//
	bsz 	= rxbuf[2] + 3;
	bcnt	= 3;
	cntrg	= 0;
	while(bcnt < bsz)
	{
		reg = (u_int16_t)(rxbuf[bcnt]) << 8;
		bcnt++;
		reg |= (u_int16_t)(rxbuf[bcnt]);
		bcnt++;
		(*(u_int16_t*)(Reg + cntrg)) = reg;
		cntrg++;
	}
	//rcount = read(fd, &rxbuf[0], _szbuf);
	usleep(wait_tx_rx);
	//
	return _no_err;
}

//команда 6 - запись в один регистр
int MRTU_CMD_6(u_int8_t Dev, u_int16_t Addr, u_int16_t Reg)
{
	u_int16_t		crc;
	u_int16_t		incrc;
	int 		wcount;
	int 		rcount;
	//
	txbuf[0]	= Dev;	//адрес
	txbuf[1]	= 6;	//команда
	txbuf[2]	= (u_int8_t)((Addr & 0xFF00) >> 8);
	txbuf[3]	= (u_int8_t)(Addr & 0xFF);
	txbuf[4]	= (u_int8_t)((Reg & 0xFF00) >> 8);
	txbuf[5]	= (u_int8_t)(Reg & 0xFF);
	//
	crc 		= MRTU_CRC(&txbuf[0], 6);
	txbuf[6]	= (u_int8_t)((crc >> 8) & 0xFF);
	txbuf[7]	= (u_int8_t)(crc & 0xFF);
	//
	tcflush(fd, TCIOFLUSH);
	wcount = write(fd, &txbuf[0], 8);
    	if (wcount < 8) {
		#if(_mdbg == 1)
        		printf("send CMD6 error! ...dev: %i  addr: %i\n", Dev, Addr);
		#endif
        	return _tx_err;
    	}
	//tcflush(fd, TCIOFLUSH);
	usleep(wait_tx_rx);
	memset(&rxbuf[0], 0, _szbuf);
	rcount = read(fd, &rxbuf[0], _szbuf);
    	if (rcount < 1)
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD6 error! ...dev: %i  addr: %i\n", Dev, Addr);
		#endif
        	return _rx_err;
	}
	crc		= MRTU_CRC(&rxbuf[0], rcount - 2);
	incrc 		= (u_int16_t)((u_int8_t)(rxbuf[rcount - 2])) << 8;
	incrc 		|= (u_int16_t)((u_int8_t)(rxbuf[rcount - 1]));
	if (crc != incrc)
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD6 CRC error! ...dev: %i  addr: %i  rcount: %i  CRC: %04X  INCRC: %04X\n", 
			Dev, Addr, rcount, crc, incrc);
		#endif
        	return _rx_crc_err;
	}
	if(txbuf[1] != (rxbuf[1] & 0x7F))
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD6 no main CMD! ...dev: %i  addr: %i sendCMD: %02X recvCMD: %02X\n", 
				Dev, Addr, txbuf[1], (rxbuf[1] & 0x7F));
			MRTU_TestRxTx(&txbuf[0], 8, &rxbuf[0], rcount);
		#endif
		return _rx_cmd_err;
	}
	//rcount = read(fd, &rxbuf[0], _szbuf);
	usleep(wait_tx_rx);
	//
	return _no_err;
}

//команда 16 - запись в несколько регистров
int MRTU_CMD_16(u_int8_t Dev, u_int16_t Addr, u_int16_t * Reg, u_int16_t CntReg)
{
	int		wcount;
	int 		rcount;
	u_int16_t		crc;
	u_int16_t		incrc;
	u_int8_t		cntbytes 	= (u_int8_t)(CntReg * 2);
	u_int8_t		addr 		= 0;
	u_int16_t		regs		= 0;
	txbuf[0]	= Dev;	//адрес
	txbuf[1]	= 16;	//команда
	txbuf[2]	= (u_int8_t)((Addr & 0xFF00) >> 8);
	txbuf[3]	= (u_int8_t)(Addr & 0xFF);
	txbuf[4]	= (u_int8_t)((CntReg & 0xFF00) >> 8);
	txbuf[5]	= (u_int8_t)(CntReg & 0xFF);
	txbuf[6]	= cntbytes;
	//
	while(addr < cntbytes)
	{
		txbuf[addr + 7] = (u_int8_t)(((*(u_int16_t*)(Reg + regs)) & 0xFF00)>> 8);
		addr++;
		txbuf[addr + 7] = (u_int8_t)((*(u_int16_t*)(Reg + regs)) & 0x00FF);
		addr++;
		regs++;
	}	
	//
	crc				= MRTU_CRC(&txbuf[0], cntbytes + 7);
	txbuf[addr + 7]			= (u_int8_t)((crc >> 8) & 0xFF);
	txbuf[addr + 8]			= (u_int8_t)(crc & 0xFF);
	//
	tcflush(fd, TCIOFLUSH);
	wcount = write(fd, &txbuf[0], addr + 9);
	if (wcount < (addr + 9)) {
		#if(_mdbg == 1)
        		printf("send CMD16 error! ...dev: %i  addr: %i\n", Dev, Addr);
		#endif
        	return _tx_err;
    	}
	//tcflush(fd, TCIOFLUSH);
	usleep(wait_tx_rx);
	memset(&rxbuf[0], 0, _szbuf);
	fcntl(fd, F_SETFL, FNDELAY);
	rcount = read(fd, &rxbuf[0], _szbuf);
	if (rcount < 1)
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD16 error! ...dev: %i  addr: %i\n", Dev, Addr);
		#endif
        	return _rx_err;
	}
	crc		= MRTU_CRC(&rxbuf[0], rcount - 2);
	incrc 		= (u_int16_t)((u_int8_t)(rxbuf[rcount - 2])) << 8;
	incrc 		|= (u_int16_t)((u_int8_t)(rxbuf[rcount - 1]));
	if (crc != incrc)
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD16 CRC error! ...dev: %i  addr: %i  rcount: %i  CRC: %04X  INCRC: %04X\n", 
			Dev, Addr, rcount, crc, incrc);
		#endif
        	return _rx_crc_err;
	}
	//проверяем совпадение команд исключая ошибки их выполнения
	if(txbuf[1] != (rxbuf[1] & 0x7F))
	{
		#if(_mdbg == 1)
        		printf("receive ansver CMD16 no main CMD! ...dev: %i  addr: %i sendCMD: %02X recvCMD: %02X\n", 
				Dev, Addr, txbuf[1], (rxbuf[1] & 0x7F));
			MRTU_TestRxTx(&txbuf[0], addr + 9, &rxbuf[0], rcount);
		#endif
		return _rx_cmd_err;
	}
	//rcount = read(fd, &rxbuf[0], _szbuf);
	usleep(wait_tx_rx);
	//
	return _no_err;
}

