//mb12xx.c

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include "mb12xx.h"
#include "i2c.h"
/*
******************************************************************************
Изменения:
	04.05.2017	- мелкие коррекции
	05.05.2017	- функции радиуса "пятна" отражения
******************************************************************************
*/

/*
Адрес по умолчанию
default i2c_addr = 0x70 (hex)
*/
//радиус для MB1242 по диаграмме A для 3.3V в сантиметрах
#define	_rMB1242_A33st	10	//один шаг в сантиметрах
#define	_rMB1242_A33sz	70	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_A33[] = 
{
	0,	//0 
	10, 	//10
	15,	//20
	15,	//30
	15,	//40
	15,	//50
	15,	//60
	14	//70
};

//радиус для MB1242 по диаграмме B для 3.3V в сантиметрах
#define	_rMB1242_B33st	10	//один шаг в сантиметрах
#define	_rMB1242_B33sz	120	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_B33[] = 
{
	0,	//0 
	10, 	//10
	12,	//20
	15,	//30
	20,	//40
	25,	//50
	30,	//60
	25,	//70
	20,	//80
	17,	//90
	15,	//100
	10,	//110
	0	//120
};

//радиус для MB1242 по диаграмме A для 5V в сантиметрах
#define	_rMB1242_A50st	10	//один шаг в сантиметрах
#define	_rMB1242_A50sz	90	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_A50[] = 
{
	0,	//0 
	10, 	//10
	15,	//20
	15,	//30
	15,	//40
	15,	//50
	15,	//60
	14,	//70
	10,	//80
	0	//90
};

//радиус для MB1242 по диаграмме B для 5V в сантиметрах
#define	_rMB1242_B50st	10	//один шаг в сантиметрах
#define	_rMB1242_B50sz	140	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_B50[] = 
{
	0,	//0 
	10, 	//10
	12,	//20
	20,	//30
	15,	//40
	19,	//50
	25,	//60
	30,	//70
	30,	//80
	30,	//90
	30,	//100
	30,	//110
	30,	//120
	25,	//130
	0	//140
};

//радиус для MB1242 по диаграмме C для 3.3V в сантиметрах
#define	_rMB1242_C33st	10	//один шаг в сантиметрах
#define	_rMB1242_C33sz	140	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_C33[] = 
{
	0,	//0 
	10, 	//10
	12,	//20
	15,	//30
	20,	//40
	25,	//50
	30,	//60
	32,	//70
	35,	//80
	40,	//90
	45,	//100
	35,	//110
	30,	//120
	20,	//130
	0	//140
};

//радиус для MB1242 по диаграмме C для 5V в сантиметрах
#define	_rMB1242_C50st	10	//один шаг в сантиметрах
#define	_rMB1242_C50sz	210	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_C50[] = 
{
	0,	//0 
	10, 	//10
	15,	//20
	20,	//30
	25,	//40
	27,	//50
	30,	//60
	35,	//70
	37,	//80
	39,	//90
	41,	//100
	44,	//110
	46,	//120
	47,	//130
	49,	//140
	50,	//150
	50,	//160
	50,	//170
	40,	//180
	35,	//190
	30,	//200
	0	//210
};

//радиус для MB1242 по диаграмме D для 3.3V в сантиметрах
#define	_rMB1242_D33st	10	//один шаг в сантиметрах
#define	_rMB1242_D33sz	200	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_D33[] = 
{
	0,	//0
	10,	//10
	12,	//20
	15,	//30
	20,	//40
	25,	//50
	30,	//60
	30,	//70
	30,	//80
	30,	//90
	30,	//100
	30,	//110
	30,	//120
	30,	//130
	30,	//140
	30,	//150
	30,	//160
	30,	//170
	30,	//180
	30,	//190
	30	//200
};

//радиус для MB1242 по диаграмме D для 5V в сантиметрах
#define	_rMB1242_D50st	10	//один шаг в сантиметрах
#define	_rMB1242_D50sz	200	//максимальная дистанция в сантиметрах
const u_int8_t _rMB1242_D50[] = 
{
	0,	//0
	10,	//10
	12,	//20
	15,	//30
	20,	//40
	25,	//50
	30,	//60
	30,	//70
	30,	//80
	30,	//90
	30,	//100
	30,	//110
	30,	//120
	30,	//130
	30,	//140
	30,	//150
	30,	//160
	30,	//17005.05.2017	- функции радиуса "пятна" отражения
	30,	//180
	30,	//190
	30	//200
};

int	HND;

/*
=== чтение данных сонара ===
возвращаемый параметр:
значения больше 0 - расстояние в сантиметрах (при испытаниях расстояние от 20 см - минимальное измеряемое расстояние)
значение -1 - ошибка открытия интерфейса как файла
значение -2 - ошибка открытия устройства с указанным адресом ведомого по I2C
значение -3 - ошибка передачи данных (записи) в указанный регистр
значение -4 - ошибка передачи команды чтения из указанного регистра
значение -5 - ошибка приема данных в процессе чтения
*/
int	mb1242read_sensor(u_int8_t i2c_addr)
{
	u_int8_t	dat[8];		//массив для приёма данных
	int		res;
	int		dist;
	int		hndl;
	//открываем I2C
	hndl = i2c_OPEN(i2c_addr);
	if(hndl < 0)
		return hndl;
	//подаём команду запуска
	dat[0]	= 81;
	res = i2c_WRIETE(hndl, i2c_addr, &dat[0], 1);
	i2c_CLOSE(hndl);
	if(res < 0)
		return res;
	//ожидаем сонар
	usleep(_wait_sensor);
	hndl = i2c_OPEN(i2c_addr);
	res = i2c_READ(hndl, i2c_addr, &dat[0], 2);
	i2c_CLOSE(hndl);
	//формируем дистанцию в сантиметрах
	dist	= ((dat[0] << 8) | dat[1]) & 0x7FFF;
	return dist;
}

//определение радиуса пятна
inline u_int8_t	mb1242_radius(u_int16_t	dist,		//дистанция
			u_int8_t * 	diagramm,	//массив диаграммы
			u_int16_t	maxdist,	//максимальная дистанция
			u_int8_t	stepdist)	//шаг дистанции
{
	float		kdelta;
	u_int8_t	frad1, frad2;
	u_int8_t	radius;
	u_int8_t	addr;
	int		daddr;
	if(dist >= maxdist)
	{
		addr = maxdist / stepdist;
		radius	= (*(u_int8_t*)(diagramm + addr));
	}
	else
	{
		addr = dist / stepdist;
		daddr = dist - (addr * stepdist);
		if(daddr == 0)
			radius	= (*(u_int8_t*)(diagramm + addr));
		else
		{
			frad1 	= (*(u_int8_t*)(diagramm + addr));
			frad2 	= (*(u_int8_t*)(diagramm + addr + 1));
			kdelta	= (float)(frad2 - frad1) / (float)(stepdist);
			radius	= kdelta * daddr + frad1;
		}
	}
	return	radius;
}

/*
=== чтение данных сенсора с учётом "радиуса пятна" ===
возвращаемый параметр:
значения больше 0 - расстояние в сантиметрах (при испытаниях расстояние от 20 см - минимальное измеряемое расстояние)
значение -1 - ошибка открытия интерфейса как файла
значение -2 - ошибка открытия устройства с указанным адресом ведомого по I2C
значение -3 - ошибка передачи данных (записи) в указанный регистр
значение -4 - ошибка передачи команды чтения из указанного регистра
значение -5 - ошибка приема данных в процессе чтения
*/
int	mb1242read_sensorR(u_int8_t i2c_addr, SENS_RADIUS * dat)
{
	int	res = mb1242read_sensor(i2c_addr);
	switch(dat->type_diagramm)
	{
		case _type_diagramm_MB1242_A33:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_A33[0]), _rMB1242_A33sz, _rMB1242_A33st);
			break;
		case _type_diagramm_MB1242_A50:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_A50[0]), _rMB1242_A50sz, _rMB1242_A50st);
			break;
		case _type_diagramm_MB1242_B33:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_B33[0]), _rMB1242_B33sz, _rMB1242_B33st);
			break;
		case _type_diagramm_MB1242_B50:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_B50[0]), _rMB1242_B50sz, _rMB1242_B50st);
			break;
		case _type_diagramm_MB1242_C33:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_C33[0]), _rMB1242_C33sz, _rMB1242_C33st);
			break;
		case _type_diagramm_MB1242_C50:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_C50[0]), _rMB1242_C50sz, _rMB1242_C50st);
			break;
		case _type_diagramm_MB1242_D33:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_D33[0]), _rMB1242_D33sz, _rMB1242_D33st);
			break;
		case _type_diagramm_MB1242_D50:
			dat->radius = mb1242_radius(res, (u_int8_t*)(&_rMB1242_D50[0]), _rMB1242_D50sz, _rMB1242_D50st);
			break;
		default:
			break;
	}
	return res;
}


/*
=== чтение данных сонара в метрической системе ===
возвращаемый параметр:
значения больше 0 - расстояние в метрах (при испытаниях расстояние от 0.2 м - минимальное измеряемое расстояние)
значение -1.0 - ошибка открытия интерфейса как файла
значение -2.0 - ошибка открытия устройства с указанным адресом ведомого по I2C
значение -3.0 - ошибка передачи данных (записи) в указанный регистр
значение -4.0 - ошибка передачи команды чтения из указанного регистра
значение -5.0 - ошибка приема данных в процессе чтения
*/
float	mb1242read_sensorM(u_int8_t i2c_addr)
{
	float	fres;
	int	res = mb1242read_sensor(i2c_addr);
	fres	= (float)(res) * 0.01;
	return fres;
}
