//bmpfunc.c

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include "bmp.h"
#include "wsfunc.h"
#include "wsserver.h"
//#include "msem.h"

pthread_mutex_t         	BMPmutex;
pthread_t 			thBMP;
//uint8_t				workdump[_BMP_DUMP_SIZE_];
uint8_t				flagNew;
/*
Определяем параметры файла BMP
и формируем рабочие параметры

входные параметры - имя шаблонного файла BMP
*/
int dumpBMPinit(char *fname, uint8_t *bd)
{
	int res;
	pthread_mutex_lock(&BMPmutex);
	res = BMP_FileAnalise(fname, bd);
	pthread_mutex_unlock(&BMPmutex);
	return res;
}

/*
запись пикселя в буфер BMP

входные параметры:
x,y	- координыты
R,G,B	- параметры цвета пикселя
*/
int setpixel(int16_t x, int16_t y, uint8_t R, uint8_t G, uint8_t B)
{
	int res;
	pthread_mutex_lock(&BMPmutex);
	res = BMP_SetPixel(x, y, R, G, B);
	pthread_mutex_unlock(&BMPmutex);
	return res;
}

/*
формирование линии в буфере BMP

входные параметры:
x1,y1,x2,y2	- координаты конечных точек линии
R,G,B		- параметры цвета линии
*/
int line(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B)
{
	int res = 0;
	pthread_mutex_lock(&BMPmutex);
	BMP_Line(x1, y1, x2, y2, R, G, B);
	pthread_mutex_unlock(&BMPmutex);
	return res;
}

/*
формирование окружности в буфер BMP

входные параметры:
x,y		- координаты центра окружности
radius		- радиус окружности
R,G,B		- параметры цветаuint8_t				flagNew;
*/
int circle(int16_t x, int16_t y, int16_t radius, uint8_t R, uint8_t G, uint8_t B)
{
	int res = 0;
	pthread_mutex_lock(&BMPmutex);
	BMP_Circle(x, y, radius, R, G, B);
	pthread_mutex_unlock(&BMPmutex);
	return res;
}

/*
формирование круга в буфере BMP

входные параметры:
x,y		- координаты центра окружности
radius		- радиус окружности
R,G,B		- параметры цвета
*/
int circlefill(int16_t x, int16_t y, int16_t radius, uint8_t R, uint8_t G, uint8_t B)
{
	int res = 0;
	pthread_mutex_lock(&BMPmutex);
	BMP_Circlefill(x, y, radius, R, G, B);
	pthread_mutex_unlock(&BMPmutex);
	return res;
}

/*
Формирование прямоугольника
Входные данные:
x1,y1	- координаты первого угла
x2,y2	- координаты противоположного угла
R,G,B	- параметры цвета
*/
int rectangle(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B)
{
	int res = 0;
	pthread_mutex_lock(&BMPmutex);
	BMP_rect(x1, y1, x2, y2, R, G, B);
	pthread_mutex_unlock(&BMPmutex);
	return res;
}

/*
Формирование закрашенного прямоугольника
Входные данные:
x1,y1	- координаты первого угла
x2,y2	- координаты противоположного угла
R,G,B	- параметры цвета
*/
int rectanglefill(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint8_t R, uint8_t G, uint8_t B)
{
	int res = 0;
	pthread_mutex_lock(&BMPmutex);
	BMP_rect_filled(x1, y1, x2, y2, R, G, B);
	pthread_mutex_unlock(&BMPmutex);
	return res;
}

/*
Формирование треугольника
Входные данные:
x1,y1	- координаты первого угла
x2,y2	- координаты второго угла
x3,y3	- координаты третьего угла
R,G,B	- параметры цвета
*/
void triangle(int16_t x1, int16_t  y1, int16_t x2, int16_t y2, int16_t x3, int16_t y3, uint8_t R, uint8_t G, uint8_t B)
{
	pthread_mutex_lock(&BMPmutex);
	BMP_triangle(x1, y1, x2, y2, x3, y3, R, G, B);
	pthread_mutex_unlock(&BMPmutex);
}


/*
*/

void setfont(int numfont)
{
	pthread_mutex_lock(&BMPmutex);
	BMP_SetFont(numfont);
	pthread_mutex_unlock(&BMPmutex);
}

/*
Вывод символа на экран
Входные данные:
x,y	- координаты символа
FC	- цвет символа
BC	- цвет фона
FBC	- флаг наличия фона символа при значении > 0
*/

void outchar(char sym, int16_t x, int16_t y, uint8_t R1,  uint8_t G1, uint8_t B1, uint8_t R2, uint8_t G2, uint8_t B2, uint8_t FBC)
{
	pthread_mutex_lock(&BMPmutex);
	BMP_outchar(sym, x, y, R1, G1, B1, R2, G2, B2, FBC);
	pthread_mutex_unlock(&BMPmutex);
}

/*
Вывод строки на экран
Входные данные:
x,y	- координаты начала чтроки
FC	- цвет символа
BC	- цвет фона
FBC	- флаг наличия фона при значении > 0
*/
void textout(char *str, int16_t x, int16_t y, uint8_t RF, uint8_t GF, uint8_t BF, uint8_t RB, uint8_t GB, uint8_t BB,  uint8_t FB)
{
	pthread_mutex_lock(&BMPmutex);
	BMP_textout(str, x, y, RF, GF, BF, RB, GB, BB, FB);
	pthread_mutex_unlock(&BMPmutex);
}


/*
формирование данных для отсылки картинки для передачи
по websocket
wsd	- 
*/
int wsscreen(uint8_t * wsd, uint8_t *bd, void *mt)
{
	int64_t	wsz 	= 0;
	uint8_t	flg	= 0;
	uint8_t	dbyte;
	uint8_t	*ad;
	if(flagNew)
	{
		wsz = (int64_t)(bhfi.bfSize);	//размер файла BMP
		(*(uint8_t*)(wsd))	= WS_OPCODE_BINARY | FIN1;
		if(wsz < 126)
		{
			int8_t	cnt8 	= 0;
			(*(uint8_t*)(wsd + 1))	= (uint8_t)(wsz);
			pthread_mutex_lock(&BMPmutex);
			//wmutlock(mt);
			while(cnt8 < wsz)
			{
				dbyte  = (*(uint8_t*)(bd + cnt8));
				ad	= cnt8 + wsd + 2;
				if(dbyte != (*(uint8_t*)(ad)))
				{
					flg = 0xFF;
					(*(uint8_t*)(ad))	= dbyte;
				}
				cnt8++;
			}
			pthread_mutex_unlock(&BMPmutex);
			//wmutunlock(mt);
			if(flg)
				return (wsz + 2);
		}
		else if(wsz < 65535)
		{
			int32_t	cnt16 	= 0;
			(*(uint8_t*)(wsd + 1)) 	= 126;
			(*(uint8_t*)(wsd + 2)) 	= (uint8_t)((wsz >> 8) & 0xFF);
			(*(uint8_t*)(wsd + 3)) 	= (uint8_t)(wsz & 0xFF);
			pthread_mutex_lock(&BMPmutex);
			//wmutlock(mt);
			while(cnt16 < wsz)
			{
				dbyte  = (*(uint8_t*)(bd + cnt16));
				ad	= cnt16 + wsd + 4;
				if(dbyte != (*(uint8_t*)(ad)))
				{
					flg = 0xFF;
					(*(uint8_t*)(ad))	= dbyte;
				}
				cnt16++;
			}
			pthread_mutex_unlock(&BMPmutex);
			//wmutunlock(mt);
			if(flg)
				return (wsz + 4);
		}
		else
		{
			int64_t	cnt64 	= 0;
			(*(uint8_t*)(wsd + 1))	= 127;
			//формирование размера
			(*(uint8_t*)(wsd + 2))	= (uint8_t)((wsz >> 56) & 0xFF);
			(*(uint8_t*)(wsd + 3))	= (uint8_t)((wsz >> 48) & 0xFF);
			(*(uint8_t*)(wsd + 4))	= (uint8_t)((wsz >> 40) & 0xFF);
			(*(uint8_t*)(wsd + 5))	= (uint8_t)((wsz >> 32) & 0xFF);
			(*(uint8_t*)(wsd + 6))	= (uint8_t)((wsz >> 24) & 0xFF);
			(*(uint8_t*)(wsd + 7))	= (uint8_t)((wsz >> 16) & 0xFF);
			(*(uint8_t*)(wsd + 8))	= (uint8_t)((wsz >> 8) & 0xFF);
			(*(uint8_t*)(wsd + 9))	= (uint8_t)(wsz & 0xFF);
			pthread_mutex_lock(&BMPmutex);
			//wmutlock(mt);
			while(cnt64 < wsz)
			{
				dbyte   = (*(uint8_t*)(bd + cnt64));
				ad	= cnt64 + wsd + 10;
				if(dbyte != (*(uint8_t*)(ad)))
				{
					flg = 0xFF;
					(*(uint8_t*)(ad))	= dbyte;
				}
				cnt64++;
			}
			pthread_mutex_unlock(&BMPmutex);
			//wmutunlock(mt);
			if(flg)
				return (wsz + 10);
		}
	}
	return 0;
} 

/*pthread_mutex_lock(&BMPmutex);
 ****************************
 * поток перезаписи файла *
 ****************************
 */
void bmpredraw(char * nfile)
{
	//FILE * file;
	//char	filename[256]	= {0};
	//strcpy(&filename[0], nfile);
	printf("starting loop redraw.....\r\n");
    	while(1)
	{
		/*file = fopen(&filename[0],"wb");
    		if(file < 0) {
        		printf("Open %s  error!\n",&filename[0]);
        		goto met1;
    		}
		fseek(file, 0, 0);
            	fwrite(&BMP_DUMP[0], 1, bhfi.bfSize, file);
        	fclose(file);
        	//
		printf("---loop---\n");*/
        	usleep(_interval_redraw);
	}
//met1:	printf("stop loop redraw.....\r\n");
	//
	pthread_exit(0);
}

/*
инициализация BMP буфера из файла BMP,
работа обновления данных ф\в файле BMP
*/

int bmpfilework(char * nfile, uint8_t *bd)
{
	int res;
	pthread_mutex_init(&BMPmutex,NULL);
	res = dumpBMPinit(nfile, bd);
	//pthread_create(&thBMP, NULL, (void*)bmpredraw, nfile);
	return res;
}



	
