//buttons.c

/*
	07.06.2016
*/

#include <string.h>
#include <pthread.h>
#include "bmp.h"
#include "bmpfunc.h"
#include "buttons.h"

BUTTONS	buttons;

//отображение не нажатой кнопки
void buttonPOP(int x, int y, int w, int h, char * txt, 
	uint8_t R, uint8_t G, uint8_t B,
	uint8_t Rt, uint8_t Gt, uint8_t Bt)
{
	int 	txsz;
	int	x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old;
	uint8_t	txtlen;
	pthread_mutex_lock(&BMPmutex);
	//сохраним регион
	x1_clip_old = x1_clip;
	y1_clip_old = y1_clip;
	x2_clip_old = x2_clip;
	y2_clip_old = y2_clip;
	//установим регион
	BMP_GSetClip(x, y, x + w, y + h);
	//отрисовка кнопки 
	BMP_rect(x, y, x + w, y + h, R/8, G/8, B/8);
	BMP_rect(x + 1, y + 1, x + w -1, y + h - 1, R/4, G/4, B/4);
	BMP_rect(x + 2, y + 2, x + w -2, y + h -2, R/2, G/2, B/2);
	BMP_rect_filled(x + 3, y + 3, x + w - 3, y + h - 3, R, G, B);
	//выбор шрифта и его параметров
	BMP_SetFont(FontNum_8_8_1);
	//выравнивание текста по центру кнопки
	txtlen = strlen(txt);
	txsz = txtlen * FontBits;
	//надпись на кнопке
	BMP_textout(txt, x + (w / 2) - (txsz / 2), y + (h / 2) - (FontByts / 2), Rt, Gt, Bt, 0, 0, 0, 0);
	//восстановим регион
	BMP_GSetClip(x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old);
	pthread_mutex_unlock(&BMPmutex);
	
}

//отображение нажатой кнопки
void buttonPUSH(int x, int y, int w, int h, char * txt, 
	uint8_t R, uint8_t G, uint8_t B,
	uint8_t Rt, uint8_t Gt, uint8_t Bt)
{
	int 	txsz;
	int	x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old;
	uint8_t	txtlen;
	pthread_mutex_lock(&BMPmutex);
	//сохраним регион
	x1_clip_old = x1_clip;
	y1_clip_old = y1_clip;
	x2_clip_old = x2_clip;
	y2_clip_old = y2_clip;
	//установим регион
	BMP_GSetClip(x, y, x + w, y + h);
	//отрисовка кнопки 
	BMP_rect_filled(x, y, x + w, y + h, R/2, G/2, B/2);
	//выбор шрифта и его параметров
	BMP_SetFont(FontNum_8_8_1);
	//выравнивание текста по центру кнопки
	txtlen = strlen(txt);
	txsz = txtlen * FontBits;
	//надпись на кнопке
	BMP_textout(txt, x + (w / 2) - (txsz / 2), y + (h / 2) - (FontByts / 2), Rt/2, Gt/2, Bt/2, 0, 0, 0, 0);
	//восстановим регион
	BMP_GSetClip(x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old);
	pthread_mutex_unlock(&BMPmutex);
	
}

//инициализация структуры кнопок
void buttonsINIT(void)
{
	memset(&buttons, 0, sizeof(BUTTONS));
}

//формирование кнопки
int buttonCREATE(int x, int y, int w, int h, char * txt, uint8_t R, uint8_t G, uint8_t B, uint8_t Rt, uint8_t Gt, uint8_t Bt, void (*f)())
{
	int	cnt = 0;
	int	sztxt;
	while(cnt < _CNTBT)
	{
		//поиск свободных мест в структуре
		if(!(buttons.button[cnt].used))
		{
			sztxt = strlen(txt);
			if(!(sztxt < _SZTXT))
				return -1;
			//заполнение данных кнопки
			buttons.button[cnt].used 	= 0xFF;
			buttons.button[cnt].x		= x;
			buttons.button[cnt].y		= y;
			buttons.button[cnt].w		= w;
			buttons.button[cnt].h		= h; 
			strcpy(&buttons.button[cnt].txt[0], txt);
			buttons.button[cnt].R		= R;
			buttons.button[cnt].G		= G;
			buttons.button[cnt].B		= B;
			buttons.button[cnt].Rt		= Rt;
			buttons.button[cnt].Gt		= Gt;
			buttons.button[cnt].Bt		= Bt;
			buttons.button[cnt].pf		= *f;
			//
			//buttonPOP(x, y, w, h, txt, R, G, B, Rt, Gt, Bt);
			buttonPOP(buttons.button[cnt].x, 
				buttons.button[cnt].y, 
				buttons.button[cnt].w, 
				buttons.button[cnt].h, 
				&buttons.button[cnt].txt[0], 
				buttons.button[cnt].R,
				buttons.button[cnt].G,
				buttons.button[cnt].B,
				buttons.button[cnt].Rt,
				buttons.button[cnt].Gt,
				buttons.button[cnt].Bt);
			buttons.cntb++;
			return 0;
		}
		cnt++;
	}
	return -2;
}

//проверка нажатия кнопок
uint8_t buttonCTRL(int16_t x, int16_t y, uint8_t flag)
{
	int	cnt 	= 0;
	uint8_t	ret	= 0;
	while(cnt < buttons.cntb)
	{
		if((x > buttons.button[cnt].x) &&
		(x < (buttons.button[cnt].x + buttons.button[cnt].w)) &&
		(y > buttons.button[cnt].y) &&
		(y < (buttons.button[cnt].y + buttons.button[cnt].h)) &&
		(flag))
		{
			buttonPUSH(buttons.button[cnt].x, buttons.button[cnt].y, 
			buttons.button[cnt].w, buttons.button[cnt].h, 
			&buttons.button[cnt].txt[0], 
			buttons.button[cnt].R, buttons.button[cnt].G, buttons.button[cnt].B, 
			buttons.button[cnt].Rt, buttons.button[cnt].Gt, buttons.button[cnt].Bt);
			flag = 0;
			if(buttons.button[cnt].pf != 0)
				((*buttons.button[cnt].pf)());
			ret	= 0xFF;
			printf("button: ret=%i, x=%i, y=%i\n", ret, x,y);
		}
		else 
		{
			buttonPOP(buttons.button[cnt].x, buttons.button[cnt].y, 
			buttons.button[cnt].w, buttons.button[cnt].h, 
			&buttons.button[cnt].txt[0], 
			buttons.button[cnt].R, buttons.button[cnt].G, buttons.button[cnt].B, 
			buttons.button[cnt].Rt, buttons.button[cnt].Gt, buttons.button[cnt].Bt);
		}
		cnt++;
	}
	return ret;
}


