//slider.c

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <string.h>
#include "bmp.h"
#include "slider.h"

SLIDERS 	sliders;

//отображение горизонтального слайдера
__inline void sliderH(int16_t x, int16_t y, int16_t w, int16_t h,
	int16_t min, float k, int16_t cur,
	uint8_t RF, uint8_t GF, uint8_t BF,
	uint8_t RB, uint8_t GB, uint8_t BB)
{
	int16_t	x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old;
	int16_t	X, szX;
	int16_t	h_4	= h/4;
	int16_t	h_2	= h/2;
	if(w < 20)
		szX = 4;
	else if(w < 51)
		szX = 6;
	else
		szX = 8;
	pthread_mutex_lock(&BMPmutex);
	//сохраним регион
	x1_clip_old = x1_clip;
	y1_clip_old = y1_clip;
	x2_clip_old = x2_clip;
	y2_clip_old = y2_clip;
	//установим регион
	BMP_GSetClip(x, y, x + w, y + h);
	//отрисовка фона
	BMP_rect_filled(x, y, x + w, y + h, 0, 0, 0);
	BMP_rect(x, y + h_4, x + w - 1, y + h - h_4, (RF >> 1), (GF >> 1), (BF >> 1));
	BMP_rect(x + 1, y + h_4 + 1, x + w - 2, y + h - h_4 - 1, RF, GF, BF);
	BMP_rect_filled(x + 2, y + h_4 + 2, x + w - 3, y  + h - h_4 - 2, RB, GB, BB);
	BMP_rect_filled(x + 3, y + h_2 - 1, x + w - 4, y + h_2 + 1, (RB >> 2), (GB >> 2), (BB >> 2));
	//отрисовка ползунка
	X = x + (int16_t)((float)(cur - min) / k);
	if(X > (x + w - szX))
		X = (x + w - szX);
	else if(X < x)
		X = x;
	//BMP_rect_filled(X, y, szX, h, RF, GF, BF);
	BMP_rect_filled(X, y, X + szX/2, y + h, RF, GF, BF);

	//восстановим регион
	BMP_GSetClip(x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old);
	pthread_mutex_unlock(&BMPmutex);
}

//отображение вертикального слайдера
__inline void sliderV(int16_t x, int16_t y, int16_t w, int16_t h,	//позиция и размер
	//int16_t min, int16_t max, int16_t cur,	//мин. макс. тек.
	int16_t min, float k, int16_t cur,
	uint8_t RF, uint8_t GF, uint8_t BF,
	uint8_t RB, uint8_t GB, uint8_t BB)
{
	int16_t	x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old;
	int16_t	Y, szY;
	int16_t w_4	= w/4;
	int16_t w_2	= w/2;
	//
	if(h < 20)
		szY = 4;
	else if(h < 51)
		szY = 6;
	else
		szY = 8;
	pthread_mutex_lock(&BMPmutex);
	//сохраним регион
	x1_clip_old = x1_clip;
	y1_clip_old = y1_clip;
	x2_clip_old = x2_clip;
	y2_clip_old = y2_clip;
	//установим регион
	BMP_GSetClip(x, y, x + w, y + h);
	//отрисовка фона
	BMP_rect_filled(x, y, x + w, y + h, 0, 0, 0);
	BMP_rect(x + w_4, y, x + w - w_4, y + h, (RF >> 1), (GF >> 1), (BF >> 1));
	BMP_rect(x + w_4 + 1, y + 1, x + w - w_4 - 1, y + h - 1, RF, GF, BF);
	BMP_rect_filled(x + w_4 + 2, y + 2, x + w - w_4 - 2, y + h - 2, RB, GB, BB);
	BMP_rect_filled(x + w_2 - 1, y + 3, x + w_2 + 1, y + h - 3, (RB >> 2), (GB >> 2), (BB >> 2));
	//отрисовка ползунка
	Y = y + (int16_t)((float)(cur - min) / k);
	if(Y > (y + h - szY))
		Y = (y + h - szY);
	else if(Y < y)
		Y = y;
	//BMP_rect_filled(x, Y, w, szY, RF, GF, BF);
	BMP_rect_filled(x, Y, x + w, Y + szY/2, RF, GF, BF);
	//восстановим регион
	BMP_GSetClip(x1_clip_old, y1_clip_old, x2_clip_old, y2_clip_old);
	pthread_mutex_unlock(&BMPmutex);
}

//инициализация структуры слайдеров
void slidersINIT(void)
{
	memset(&sliders, 0, sizeof(SLIDERS));
}

//формирование слайдера
int sliderCREATE(int16_t x, int16_t y, int16_t w, int16_t h,
		int16_t min, int16_t max, int16_t * par,
		uint8_t RF, uint8_t GF, uint8_t BF,
		uint8_t RB, uint8_t GB, uint8_t BB)
{
	int 		cnt = 0;
	while(cnt < _CNTSLDR)
	{	if(!(sliders.slider[cnt].used))
		{
			sliders.slider[cnt].used	= 0xFF;
			//определение вертикального флага
			if(h > w)
				sliders.slider[cnt].fv = 0xFF;
			sliders.slider[cnt].x	= x;
			sliders.slider[cnt].y	= y;
			sliders.slider[cnt].w	= w;
			sliders.slider[cnt].h	= h;
			sliders.slider[cnt].min	= min;
			sliders.slider[cnt].max	= max;
			sliders.slider[cnt].val	= par;
			sliders.slider[cnt].RF	= RF;
			sliders.slider[cnt].GF	= GF;
			sliders.slider[cnt].BF	= BF;
			sliders.slider[cnt].RB	= RB;
			sliders.slider[cnt].GB	= GB;
			sliders.slider[cnt].BB	= BB;
			//
			sliders.slider[cnt].delta = sliders.slider[cnt].max - sliders.slider[cnt].min;
			if(sliders.slider[cnt].fv)
			{
				sliders.slider[cnt].k =  (float)(sliders.slider[cnt].delta) / (float)(sliders.slider[cnt].h);	//vert
				sliderV(x, y, w, h, min, sliders.slider[cnt].k, *par, RF, GF, BF, RB, GB, BB);
			}
			else
			{
				sliders.slider[cnt].k =  (float)(sliders.slider[cnt].delta) / (float)(sliders.slider[cnt].w);	//hor
				sliderH(x, y, w, h, min, sliders.slider[cnt].k, *par, RF, GF, BF, RB, GB, BB);
			}
			sliders.cntslrd++;
			return 0;
		}
		cnt++;
	}
	return -2;
}

uint8_t sliderCTRL(int16_t x, int16_t y, uint8_t flag)
{
	int 		cnt	= 0;
	int16_t		par;
	uint8_t		ret	= 0;
	while(cnt < sliders.cntslrd)
	{
		//проверка координат
		if((x > sliders.slider[cnt].x) &&
		(x < (sliders.slider[cnt].x + sliders.slider[cnt].w)) &&
		(y > sliders.slider[cnt].y) &&
		(y < (sliders.slider[cnt].y + sliders.slider[cnt].h)) &&
		(flag))
		{
			if(sliders.slider[cnt].fv)
				par = (int16_t)((float)(y - sliders.slider[cnt].y) * sliders.slider[cnt].k) + sliders.slider[cnt].min;	//vert
			else
				par = (int16_t)((float)(x - sliders.slider[cnt].x) * sliders.slider[cnt].k) + sliders.slider[cnt].min;	//hor
			(*(int16_t*)(sliders.slider[cnt].val)) = par;
			flag	= 0;
			ret	= 0xFF;
			printf("slider: ret=%i, x=%i, y=%i\n", ret, x,y);
		}
		//
		par = (*(int16_t*)(sliders.slider[cnt].val));
		if(sliders.slider[cnt].fv)
		{
			//printf("V: min=%i delta=%i k=%f cur=%i\n", sliders.slider[cnt].min, sliders.slider[cnt].delta, sliders.slider[cnt].k, par);
			sliderV(sliders.slider[cnt].x,
			sliders.slider[cnt].y,
			sliders.slider[cnt].w,
			sliders.slider[cnt].h,
			sliders.slider[cnt].min,
			sliders.slider[cnt].k,
			par,
			sliders.slider[cnt].RF,
			sliders.slider[cnt].GF,
			sliders.slider[cnt].BF,
			sliders.slider[cnt].RB,
			sliders.slider[cnt].GB,
			sliders.slider[cnt].BB);
		}
		else
		{
			sliderH(sliders.slider[cnt].x,
			sliders.slider[cnt].y,
			sliders.slider[cnt].w,
			sliders.slider[cnt].h,
			sliders.slider[cnt].min,
			sliders.slider[cnt].k,
			par,
			sliders.slider[cnt].RF,
			sliders.slider[cnt].GF,
			sliders.slider[cnt].BF,
			sliders.slider[cnt].RB,
			sliders.slider[cnt].GB,
			sliders.slider[cnt].BB);
		}	
		cnt++;
	}
	return ret;
}

	

