//swfunc.c

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>    
#include <time.h>
#include <pthread.h>
#include <signal.h>
#include "myws.h"
#include "wsserver.h"
#include "wsfunc.h"
#include "wsconfig.h"
#include <openssl/sha.h>
#include "base64.h"

uint8_t		rdbuf[_BUFSIZEin];

/* === выборка значений ключей из строк ===
inbuf  - исходная строка для разбора
inref  - имя ключа в исходной строке
output - вывод значения ключа
!!! - имя ключа и его значение в исходной строке засеняются 
пробелами
*/
__inline int strval(char * inbuf, char * inref, char * output)
{
	char		* p;
	uint8_t		m = 0;
	uint8_t		n = 0;
	uint8_t		kflag = 0;
	if((p = strstr(inbuf, inref)) != NULL)
	{
		while(1)
		{
			if(((*(char*)(p + m)) == '\n') ||
			((*(char*)(p + m)) == '\r'))
				break;
			else if((*(char*)(p + m)) == ' ')
				kflag = 0xFF;
			else if(kflag)
			{
				(*(char*)(output + n)) = (*(char*)(p + m));
				n++;
			}
			(*(char*)(inbuf + m)) = ' ';
			m++;
		}
		return 1;
	}
	else
		return 0;
}

/* === выделение параметров запроса === */
int wsin(WSDAT * wsd)
{
	int res;
	char	outstr[64];
	#if(_dbg_msg_wsin > 0)
	printf("Rd:\n%s\n", (char*)(wsd->inbuffer));
	printf("=== wsin ===\n");
	#endif
	memset(wsd->Sec_WebSocket_Key, 0, sizeof(wsd->Sec_WebSocket_Key));
	res = strval((char*)(wsd->inbuffer), "Sec-WebSocket-Key:", (char*)(wsd->Sec_WebSocket_Key));
	#if(_dbg_msg_wsin > 0)
	if(res)
		printf("Sec-WebSocket-Key: %s\n", wsd->Sec_WebSocket_Key);
	#endif
	memset(&outstr[0], 0, 64);
	res = strval((char*)(wsd->inbuffer), "Sec-WebSocket-Version:", (char*)(&outstr[0]));
	wsd->Sec_WebSocket_Version	= atoi(&outstr[0]);
	#if(_dbg_msg_wsin > 0)
	if(res)
		printf("Sec-WebSocket-Version: %i\n", wsd->Sec_WebSocket_Version);
	#endif
	memset(wsd->Sec_WebSocket_Extensions, 0, sizeof(wsd->Sec_WebSocket_Extensions));
	res = strval((char*)(wsd->inbuffer), "Sec-WebSocket-Extensions:", (char*)(wsd->Sec_WebSocket_Extensions));
	#if(_dbg_msg_wsin > 0)
	if(res)
		printf("Sec-WebSocket-Extensions: %s\n", wsd->Sec_WebSocket_Extensions);
	#endif
	//
	return res;
}

/* === формирование данных для отправки === */
int wsout(WSDAT * wsd)
{
	char		outstr[64];
	char		temp[_SHA_DIGEST_LENGTH] = {0};
	//SHA1_CTX	context;
	memset(&outstr, 0, sizeof(outstr));
	#if(_dbg_msg_wsout > 0)
	printf("=== wsout ===\n");
	#endif
	strcpy(&outstr[0], wsd->Sec_WebSocket_Key);
	strcat(outstr, GUIDKey);
	#if(_dbg_msg_wsout > 0)
	printf("::: %s\n", &outstr[0]);
	#endif
	//SHA1(&temp[0], &outstr[0], strlen(outstr)-1 );
	SHA1((unsigned char*)(&outstr[0]), strlen(outstr), (unsigned char*)(temp));
	#if(_dbg_msg_wsout > 0)
	printf("SHA1: %s\n", &temp[0]);
	#endif
	memset(wsd->Sec_WebSocket_Accept, 0, sizeof(wsd->Sec_WebSocket_Accept));
	Base64encode(wsd->Sec_WebSocket_Accept, temp, strlen(temp));
	#if(_dbg_msg_wsout > 0)
	printf("Key_for_client: %s\n", wsd->Sec_WebSocket_Accept);
	#endif
	memset(wsd->resp, 0, sizeof(wsd->resp));
	snprintf((char*)(wsd->resp), 130, "%s%s%s", response_ws, wsd->Sec_WebSocket_Accept, "\r\n\r\n");
	wsd->szresp = strlen((char*)(wsd->resp));
	#if(_dbg_msg_wsout > 0)
	printf("Resp(size=%i):\n%s\n", wsd->szresp, wsd->resp);
	#endif
	//
	return 0;
}

/*
секция работы с websocket
*/

int wsfunc(WSDAT * wsd)
{
	int		res;
	unsigned int	i,pl;
	uint64_t	pl64;
	uint8_t		payload_len;
	uint16_t	payload_len16	= 0;
	uint64_t	payload_len64	= 0;
	uint8_t		opcode;
	uint8_t		mask;
	memset(&rdbuf[0], 0, _BUFSIZEin);
	#if(_dbg_msg_wsfunc > 0)
	printf("=== wsfunc ===\n");
	#endif
	#if(_dbg_msg_wsfunc > 0)
	printf("===\nRd(ws):\n%s\n==\n", wsd->inbuffer);
	#endif
	opcode 		= wsd->inbuffer[0] & 0x0F;
	payload_len	= wsd->inbuffer[1] & 0x7F;
	mask		= wsd->inbuffer[1] & 0x80 ? 0xFF:0x00;
	//
	#if(_dbg_msg_wsfunc > 0)
	printf("FIN: 0x%02X\n", opcode & 0x01);
	printf("RSV1: 0x%02X\n", opcode & 0x02);
	printf("RSV2: 0x%02X\n", opcode & 0x04);
	printf("RSV3: 0x%02X\n", opcode & 0x08);
	printf("Opcode: 0x%02X\n", opcode & 0x0F);
	printf("payload_len: %i\n", payload_len);
	printf("Mask On: 0x%02X\n", mask);
	#endif
	//
	switch(opcode)
	{
		case WS_OPCODE_CONTINUE:
			#if(_dbg_msg_wsfunc > 0)
			printf("opcode = WS_OPCODE_CONTINUE\n");
			#endif
			break;
		case WS_OPCODE_TEXT:
			#if(_dbg_msg_wsfunc > 0)
			printf("opcode = WS_OPCODE_TEXT\n");
			#endif
			switch(payload_len)
			{
				case 126:
       					payload_len16 = ((uint16_t)(wsd->inbuffer[2]) << 8) | 
							(uint16_t)(wsd->inbuffer[3]);
					wsd->masking_key[0] = wsd->inbuffer[4];
					wsd->masking_key[1] = wsd->inbuffer[5];
					wsd->masking_key[2] = wsd->inbuffer[6];
					wsd->masking_key[3] = wsd->inbuffer[7];
					i = 8, pl = 0;
       	    				for(; pl < payload_len16; i++, pl++)
					{
						if(mask)
       	       						rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
				case 127:
	       				payload_len64 = ((uint64_t)(wsd->inbuffer[2]) << 56) |
							((uint64_t)(wsd->inbuffer[3]) << 48) |
							((uint64_t)(wsd->inbuffer[4]) << 40) |
							((uint64_t)(wsd->inbuffer[5]) << 32) |
							((uint64_t)(wsd->inbuffer[6]) << 24) |
							((uint64_t)(wsd->inbuffer[7]) << 16) |
							((uint64_t)(wsd->inbuffer[8]) << 8) |
							(uint64_t)(wsd->inbuffer[9]);
					wsd->masking_key[0] = wsd->inbuffer[10];
					wsd->masking_key[1] = wsd->inbuffer[11];
					wsd->masking_key[2] = wsd->inbuffer[12];
					wsd->masking_key[3] = wsd->inbuffer[13];
					i = 8, pl64 = 0;
	       	    			for(; pl64 < payload_len64; i++, pl++)
					{
						if(mask)
	       	       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
				default: 
					wsd->masking_key[0] = wsd->inbuffer[2];
					wsd->masking_key[1] = wsd->inbuffer[3];
					wsd->masking_key[2] = wsd->inbuffer[4];
					wsd->masking_key[3] = wsd->inbuffer[5];
					i = 6, pl = 0;
    					for(; pl < payload_len; i++, pl++)
					{
						if(mask)
       		       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
			}
			#if(_dbg_msg_wsfunc > 0)
			printf("\nReciv:%s\n", rdbuf);
			#endif
			//
			res = strlen((const char*)(wsd->inbuffer));
			return res;
		case WS_OPCODE_BINARY:
			#if(_dbg_msg_wsfunc > 0)
			printf("opcode = WS_OPCODE_BINARY\n");
			#endif
			switch(payload_len)
			{
				case 126:
	       				payload_len16 = ((uint16_t)(wsd->inbuffer[2]) << 8) | 
							(uint16_t)(wsd->inbuffer[3]);
					wsd->masking_key[0] = wsd->inbuffer[4];
					wsd->masking_key[1] = wsd->inbuffer[5];
					wsd->masking_key[2] = wsd->inbuffer[6];
					wsd->masking_key[3] = wsd->inbuffer[7];
					i = 8, pl = 0;
	       	    			for(; pl < payload_len16; i++, pl++)
					{
						if(mask)
	       	       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
				case 127:
	       				payload_len64 = ((uint64_t)(wsd->inbuffer[2]) << 56) |
							((uint64_t)(wsd->inbuffer[3]) << 48) |
							((uint64_t)(wsd->inbuffer[4]) << 40) |
							((uint64_t)(wsd->inbuffer[5]) << 32) |
							((uint64_t)(wsd->inbuffer[6]) << 24) |
							((uint64_t)(wsd->inbuffer[7]) << 16) |
							((uint64_t)(wsd->inbuffer[8]) << 8) |
							(uint64_t)(wsd->inbuffer[9]);
					wsd->masking_key[0] = wsd->inbuffer[10];
					wsd->masking_key[1] = wsd->inbuffer[11];
					wsd->masking_key[2] = wsd->inbuffer[12];
					wsd->masking_key[3] = wsd->inbuffer[13];
					i = 8, pl64 = 0;
	       	    			for(; pl64 < payload_len64; i++, pl++)
					{
						if(mask)
	       	       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
				default: 
					wsd->masking_key[0] = wsd->inbuffer[2];
					wsd->masking_key[1] = wsd->inbuffer[3];
					wsd->masking_key[2] = wsd->inbuffer[4];
					wsd->masking_key[3] = wsd->inbuffer[5];
					i = 6, pl = 0;
	    				for(; pl < payload_len; i++, pl++)
					{
						if(mask)
	       	       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
			}
			#if(_dbg_msg_wsfunc > 0)
			printf("\nReciv:%s\n", rdbuf);
			#endif
			break;
		case WS_OPCODE_CLOSE:
			#if(_dbg_msg_wsfunc > 0)
			printf("opcode = WS_OPCODE_CLOSE\n");
			#endif
			return -1;	//выходим и закрываем соединение
		case WS_OPCODE_PING:
			#if(_dbg_msg_wsfunc > 0)
			printf("opcode = WS_OPCODE_PING\n");
			#endif
			break;
		case WS_OPCODE_PONG:
			#if(_dbg_msg_wsfunc > 0)
			printf("opcode = WS_OPCODE_PONG\n");
			#endif
			switch(payload_len)
			{
				case 126:
	       				payload_len16 = ((uint16_t)(wsd->inbuffer[2]) << 8) | 
							(uint16_t)(wsd->inbuffer[3]);
					wsd->masking_key[0] = wsd->inbuffer[4];
					wsd->masking_key[1] = wsd->inbuffer[5];
					wsd->masking_key[2] = wsd->inbuffer[6];
					wsd->masking_key[3] = wsd->inbuffer[7];
					i = 8, pl = 0;
	       	    			for(; pl < payload_len16; i++, pl++)
					{
						if(mask)
	       	       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
				case 127:
	       				payload_len64 = ((uint64_t)(wsd->inbuffer[2]) << 56) |
							((uint64_t)(wsd->inbuffer[3]) << 48) |
							((uint64_t)(wsd->inbuffer[4]) << 40) |
							((uint64_t)(wsd->inbuffer[5]) << 32) |
							((uint64_t)(wsd->inbuffer[6]) << 24) |
							((uint64_t)(wsd->inbuffer[7]) << 16) |
							((uint64_t)(wsd->inbuffer[8]) << 8) |
							(uint64_t)(wsd->inbuffer[9]);
					wsd->masking_key[0] = wsd->inbuffer[10];
					wsd->masking_key[1] = wsd->inbuffer[11];
					wsd->masking_key[2] = wsd->inbuffer[12];
					wsd->masking_key[3] = wsd->inbuffer[13];
					i = 8, pl64 = 0;
	       	    			for(; pl64 < payload_len64; i++, pl++)
					{
						if(mask)
	       	       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
				default: 
					wsd->masking_key[0] = wsd->inbuffer[2];
					wsd->masking_key[1] = wsd->inbuffer[3];
					wsd->masking_key[2] = wsd->inbuffer[4];
					wsd->masking_key[3] = wsd->inbuffer[5];
					i = 6, pl = 0;
	    				for(; pl < payload_len; i++, pl++)
					{
						if(mask)
	       	       					rdbuf[pl] = wsd->inbuffer[i]^wsd->masking_key[pl % 4];
						else
							rdbuf[pl] = wsd->inbuffer[i];
					}
					break;
			}
			#if(_dbg_msg_wsfunc > 0)
			printf("\nReciv:%s\n", rdbuf);
			#endif
			return -2;
		default:
			#if(_dbg_msg_wsfunc > 0)
			printf("opcode = ?\n");
			#endif
			break;
	}
	return 0;
}


